/* DirectoryRecurser.java
 *
 * Copyright (C) 2000-2001 David Medinets
 * All Rights Reserved.
 *
 */
package com.affy.jarsearcher;


import com.affy.jarsearcher.commands.AddClass;
import com.affy.jarsearcher.exceptions.JarSearcherException;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

import org.prevayler.Prevayler;
import org.prevayler.implementation.NullPrevayler;
import org.prevayler.implementation.SnapshotPrevayler;

/** Finds all files in a directory tree.
 *
 * @author David Medinets
**/
public class DirectoryRecurser {
	private Prevayler prevayler = null;
	private String rootDir = null;
	private int levelsToRecurse = 999;
	private HashMap jarFiles = new HashMap();
	private String appName = null;
	
	public DirectoryRecurser() { }
   
	public void findFiles() throws JarSearcherException {
		//prevayler = new SnapshotPrevayler(new ClassManager());
		this.prevayler = new NullPrevayler(new ClassManager());
		build_stack(0, getRootDir());
	}

	/** The workhorse of this class. It does the job of reading directories and
	 * recursing down the directory tree.
	 **/
	private void build_stack(int dir_levels_deep, String p_directory_name) throws JarSearcherException {
		if (null == p_directory_name || 0 == p_directory_name.length()) {
			return;
		}

		// System.out.println(this.getClass().getName() + ".build_stack(): " + p_directory_name);

		File dir = new File(p_directory_name);

		if (false == dir.canRead()) {
			throw new JarSearcherException(this.getClass().getName() + ".build_stack(); Assertion Error: Unable to read " + p_directory_name + ".");
		}
      
		//System.out.println("DirectoryRecurser being called on " + dir);
		selectFilesFromDirectory(dir, dir_levels_deep);

		// Get list of subdirectories for recursion.
		String[] sDirs = dir.list(new FilterDirectoryOnly());
		for (int i = 0; i < sDirs.length; i++) {
            // we always recurse, but some filters won't be invoked inside subdirectories.
			dir_levels_deep++;
			build_stack(dir_levels_deep, p_directory_name + (String)System.getProperty("file.separator") + sDirs[i]);
		}
	}

	private void selectFilesFromDirectory(File dir, int dir_levels_deep) throws JarSearcherException {
		//System.out.println("inside DirectoryRecurser.selectFilesFromDirectory: " + this.prevayler);

		String curFile = null;
		String[] sFiles = null;

		if (dir_levels_deep > getLevelsToRecurse()) {
			// do nothing..we're too deep to invoke this filter.
		} else {
			sFiles = dir.list(new FilterJar());
			if (null == sFiles) {
		         throw new JarSearcherException(this.getClass().getName() + ".build_stack();Assertion Error: GifFiles is null.");
			}

			HashMap hshFileAttributes = null;
			String filePath = null;
			String emptyString = "";

			for (int i = 0; i < sFiles.length; i++) {
				hshFileAttributes = new HashMap();

				if (null != getJarFiles().get(sFiles[i].toUpperCase())) {
					System.out.println("  DUPLICATE JAR FILE: " + sFiles[i]);
					// TODO: Handle duplicate jar files if interactive, otherwise take the first one.
				} else {

					filePath = dir.toString() + (String)System.getProperty("file.separator") + sFiles[i];
					System.out.println("  Processing: " + filePath);

					JarInputStream jis = null;
					JarEntry je = null;
				
					boolean okToProcess = true;
				
					try {
						jis = new JarInputStream(new BufferedInputStream(new FileInputStream(filePath)));
					} catch (FileNotFoundException e) {
						e.printStackTrace();
						okToProcess = false;
					} catch (IOException e) {
						e.printStackTrace();
						okToProcess = false;
					}

					if (okToProcess) {
						Vector v = new Vector();

				        try {
							while( ( je = jis.getNextJarEntry() ) != null ) {
								String jarFile = sFiles[i].toUpperCase();
								String className = je.getName().replaceAll("/", ".");
	
								if (className.endsWith(".class") && className.indexOf('$') == -1 &&  null == ClassManager.findByPrimaryKey(className, jarFile)) {
									//System.out.println("Adding " + className);
									v.add(className);
								}
							}
						} catch (IOException e) {
							e.printStackTrace();
						}
		
						hshFileAttributes.put("PATH",    dir.toString());
						hshFileAttributes.put("NAME",    sFiles[i]);
						hshFileAttributes.put("SIZE",    new Integer(sFiles[i].length()));
						hshFileAttributes.put("CLASSES", v);
						getJarFiles().put(sFiles[i].toUpperCase(), hshFileAttributes);
					}
				}
			}
		} // if (dir_levels_deep > levelsToRecurse) {
	}
	
	/**
	 * Returns the rootDir.
	 * @return String
	 */
	public String getRootDir() {
		return rootDir;
	}

	/**
	 * Sets the rootDir.
	 * @param rootDir The rootDir to set
	 */
	public void setRootDir(String rootDir) {
		this.rootDir = rootDir;
	}

	/**
	 * Returns the levelsToRecurse.
	 * @return int
	 */
	public int getLevelsToRecurse() {
		return levelsToRecurse;
	}

	/**
	 * Sets the levelsToRecurse.
	 * @param levelsToRecurse The levelsToRecurse to set
	 */
	public void setLevelsToRecurse(int levelsToRecurse) {
		this.levelsToRecurse = levelsToRecurse;
	}

	/**
	 * Returns the jarFiles.
	 * @return Hashtable
	 */
	public HashMap getJarFiles() {
		return jarFiles;
	}

	/**
	 * Returns the prevayler.
	 * @return Prevayler
	 */
	public Prevayler getPrevayler() {
		return prevayler;
	}

	private class FilterJar implements FilenameFilter {
		public boolean accept(File directory, String filename) {
			boolean bFound = false;
   			File temp = new File(directory + (String)System.getProperty("file.separator") + filename);
   			return(!temp.isDirectory() && (filename.endsWith(".jar")));
		}
	}

	private class FilterDirectoryOnly extends Object implements FilenameFilter {
		public FilterDirectoryOnly() { }
  	
		// accept directory (for recursing) and GIF files.
		public boolean accept(File dir, String name) {
   			File temp = new File(dir + (String)System.getProperty("file.separator") + name);
   			return(temp.isDirectory());
		}
	}
	/**
	 * Method setAppName.
	 * @param appName
	 */
	public void setAppName(String appName) {
		this.appName = appName;
	}

}
