/*
 * TODO: Handle duplicate JAR files gracefully. Ask which one should be processed.
 *   could be an option to take the newer file?
 *   could be an option to replace information from the previous file with the current one?
 * TODO: Handle a MERGE command which adds or reloads.
 * TODO: generalize find_class to use regexp.
 * TODO: delete_application
 */

package com.affy.jarsearcher;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StreamTokenizer;
import java.text.DecimalFormat;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeMap;
import java.util.Vector;
import java.util.Collections;

import org.prevayler.Prevayler;
import org.prevayler.implementation.NullPrevayler;
import org.prevayler.implementation.SnapshotPrevayler;

import com.affy.jarsearcher.exceptions.JarSearcherException;
import com.affy.prevayler.commands.AddLookup;
import com.affy.prevayler.commands.AddObject;
import com.affy.prevayler.commands.AddString;
import com.affy.prevayler.exceptions.AlreadyExistsException;
import com.affy.prevayler.util.ObjectManager;
import com.affy.prevayler.util.StopWatch;

/**
 * @author medined
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class JarSearcher {

	SnapshotPrevayler prevayler = null;
	boolean isInteractive = true;

	/**
	 * Constructor for JarSearcher.
	 */
	public JarSearcher() {
		super();
	}

	/**
	 * JarSearcher is a command-line utility and this method shows the various
	 * options.
	 */
	public void cmdHelp() {
		System.out.println(
			"\nNote: All arguments must be surrounded by quotes.\n\n"
			+ "Command:\n"
				+ "  help: this message.\n"
				+ "  rem:  comment.\n"
				+ "  exit: ends JarSearcher.\n"
				+ "  list: displays applications\n"
				+ "  add: 'application_name' 'root_directory'\n"
		);
	}

	

	public static void main(String[] args) throws IOException, ClassNotFoundException {
		Reader inputSrc = null;

		StopWatch.add("Main");

		JarSearcher jr = new JarSearcher();
		jr.start();

		// This technique of reading from either a script file or
		// from the console was 'borrowed' from BeanShell.
        if ( args.length > 0 ) {
			String filename = args[0];

            inputSrc = new BufferedReader(new FileReader(filename));
            jr.setIsInteractive(false);
        } else {
			inputSrc = new InputStreamReader(System.in);
        }

		StreamTokenizer st = new StreamTokenizer(inputSrc);

		jr.parseStream(st);
		jr.stop();

		System.out.println("Total Elapsed Time: " + StopWatch.elapsedTime("Main") + " ms.");
	}

	/**
	 * Method start.
	 */
	private void start() throws ClassNotFoundException, IOException {
		System.out.println("JarSearcher v.01");
		System.out.println("  Type EXIT to leave the program.");
		System.out.println("  Type HELP to display a list of commands.");
	}

	/**
	 * Method stop.
	 */
	private void stop() {
	}

	public void parseStream(StreamTokenizer st) {
		Vector v = new Vector();
	    int token = 0;

	    try { 
	        // Prepare the tokenizer for Java-style tokenizing rules
    	    st.parseNumbers();
        	st.wordChars('_', '_');
        	st.eolIsSignificant(true);
    
	        // If whitespace is not to be discarded, make this call
    	    //st.ordinaryChars(0, ' ');
    
	        // These calls caused comments to be discarded
    	    st.slashSlashComments(true);
        	st.slashStarComments(true);
    
    		if (true == isInteractive)
				System.out.print("JarSearcher> ");
	        // Parse the file
	        do {
	            token = st.nextToken();
    	        switch (token) {
	        	    case StreamTokenizer.TT_NUMBER:
    	        	    // A number was found; the value is in nval
        	        	//double num = st.nval;
            	    	break;
	            	case StreamTokenizer.TT_WORD:
    	            	// A word was found; the value is in sval
        	    	    v.add(st.sval);
    	    	        break;
	    	        case '"':
    	    	        // A double-quoted string was found; sval contains the contents
        	    	    v.add(st.sval);
            	    	break;
		            case '\'':
    		            // A single-quoted string was found; sval contains the contents
        	    	    v.add(st.sval);
            		    break;
	            	case StreamTokenizer.TT_EOL:
	            		if (false == processCommands(v)) {
	            			token = StreamTokenizer.TT_EOF;
	            		}
						v.clear();
			    		if (true == isInteractive)
							System.out.print("JarSearcher> ");
	    	            // End of line character found
    	    	        break;
	    	        case StreamTokenizer.TT_EOF:
    	    	        // End of file has been reached
        	    	    break;
		            default:
		                // A regular character was found; the value is the token itself
    		            char ch = (char)st.ttype;
        		        break;
				}
	        } while (token != StreamTokenizer.TT_EOF);

	    } catch (IOException e) {
	    	e.printStackTrace();
    	}

	}

	/**
	 * Method process.
	 */
	private boolean processCommands(Vector v) {
		long tickStart = System.currentTimeMillis();
		boolean retValue = true;
		
   		if (false == isInteractive)
			System.out.println(v);
		
		Enumeration eCmd = v.elements();
		
		while (eCmd.hasMoreElements()) {
			String cmd = (String)eCmd.nextElement();
			
			if (cmd.equalsIgnoreCase("exit")) {
				retValue = false;
				
			} else if (cmd.equalsIgnoreCase("add_application")) {
				String appName = (String)eCmd.nextElement();
				String rootDir = (String)eCmd.nextElement();
				try {
					addApplication(prevayler, appName, rootDir);
				} catch (Exception e) {
					e.printStackTrace();
				}

			} else if (cmd.equalsIgnoreCase("delete_database")) {
				deleteDatabase();

			} else if (cmd.equalsIgnoreCase("delete_application")) {
				String appName = (String)eCmd.nextElement();
				deleteApplication(prevayler, appName);

			} else if (cmd.equalsIgnoreCase("help")) {
				cmdHelp();
				
			} else if (cmd.equalsIgnoreCase("find_class")) {
				String className = (String)eCmd.nextElement();
				findClass(className);

			} else if (cmd.equalsIgnoreCase("list")) {
				listApplications(prevayler);
				break;

			} else if (cmd.equalsIgnoreCase("list_jars")) {
				String appName = (String)eCmd.nextElement();
				listJarsInApplication(appName);
				break;

			} else if (cmd.equalsIgnoreCase("list_classes")) {
				String appName = (String)eCmd.nextElement();
				String jarName = (String)eCmd.nextElement();
				listClassesInJar(appName, jarName);
				break;

			} else if (cmd.equalsIgnoreCase("reload")) {
				String appName = (String)eCmd.nextElement();
				String rootDir = (String)eCmd.nextElement();
				reloadApplication(prevayler, appName, rootDir);

			} else if (cmd.equalsIgnoreCase("recover")) {
					try {
						crashRecover();
					} catch (Exception e) {
						e.printStackTrace();
					}
			
			} else if (cmd.equalsIgnoreCase("rem")) {
				break;

			} else {
				System.out.println("Unknown command: [" + cmd + "]");
				break;
			}
		}

		long tickEnd = System.currentTimeMillis();
		long tickTotal = tickEnd - tickStart;
		DecimalFormat df = new DecimalFormat();
		df.setDecimalSeparatorAlwaysShown(true);
		System.out.println("Command Completed: " + df.format(tickTotal) + " microseconds.");

		return retValue;
	}

	/**
	 * Method findClass.
	 * @param className
	 */
	private void findClass(String className) {
		// iterate over all Applications.
		// iterate over all Jar files.
		// see if the class is in the Jar file. If so, display message.
		HashMap hshApp = (HashMap)ObjectManager.getInstance();
 		TreeMap sorted_map = new TreeMap(hshApp);
		Set setApps = sorted_map.keySet();
		Iterator iApps = setApps.iterator();
		while (iApps.hasNext()) {
			String appName = (String)iApps.next();
			HashMap hshJars = (HashMap)hshApp.get(appName);
	 		TreeMap sortedJars = new TreeMap(hshJars);
			Set setJars = sortedJars.keySet();
			Iterator iJars = setJars.iterator();
			while (iJars.hasNext()) {
				String jarName = (String)iJars.next();
				HashMap hshJar = (HashMap)hshJars.get(jarName);
				String s = (String)hshJar.get("PATH");
				Vector v = (Vector)hshJar.get("CLASSES");
				if (v.contains(className)) {
					System.out.println("Found in [" + appName + "].[" + s + "/" + jarName + "].");
				}
			}
		}
		
	}


	/**
	 * Method listClassesInJar.
	 * @param appName
	 * @param jarName
	 */
	private void listClassesInJar(String appName, String jarName) {
		HashMap hshApp = (HashMap)ObjectManager.findByPrimaryKey(appName);
		HashMap hshJar = (HashMap)hshApp.get(jarName);
		Vector v = (Vector)hshJar.get("CLASSES");
		Collections.sort(v);
		Iterator i = v.iterator();
		System.out.println("\nClass List for [" + appName + "].[" + jarName + "]:");
		while (i.hasNext()) {
			System.out.println("  " + i.next());
		}
		System.out.println("");
	}


	/**
	 * Method deleteDatabase.
	 */
	private void deleteDatabase() {
		try {
			clearPrevalenceBase();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void clearPrevalenceBase() throws Exception {
		if (prevayler != null) {
			prevayler.takeSnapshot(); //Closes the open log file.
		}
		deletePrevalenceFiles(ObjectManager.prevalenceBase());
	}

	private void deletePrevalenceFiles(String directoryName) {
		File directory = new File(directoryName);
		if(!directory.exists()) return;

		File[] files = directory.listFiles(new PrevalenceFilter());

		for(int i = 0; i < files.length; ++i){
//			out("Deleting: " + files[i]);
			verify(files[i].delete(), "Unable to delete " + files[i]);
		}
	}

	private void crashRecover() throws Exception {
		if (prevayler == null) {
			prevayler = new SnapshotPrevayler(new ObjectManager(), ObjectManager.prevalenceBase());
		}
	}

	/**
	 * Method listJarsInApplication.
	 * @param appName
	 */
	private void listJarsInApplication(String appName) {
		HashMap hsh = (HashMap)ObjectManager.findByPrimaryKey(appName);
 		TreeMap sorted_map = new TreeMap(hsh);
		Set s = sorted_map.keySet();
		Iterator i = s.iterator();
		System.out.println("\nJar List for [" + appName + "]:");
		while (i.hasNext()) {
			System.out.println("  " + i.next());
		}
		System.out.println("");
	}

	/**
	 * Method reloadApplication.
	 * @param prevayler where we will execute the command
	 * @param appName application name associated with the Jar files.
	 * @param rootDir root directory of the application.
	 */
	private static void reloadApplication(
		Prevayler prevayler,
		String appName,
		String rootDir) {

		System.out.println("Reloading application '" + appName + "'");
	}


	/**
	 * Method deleteApplication.
	 * @param prevayler
	 * @param appName application name associated with the Jar files.
	 */
	private static void deleteApplication(Prevayler prevayler, String appName) {
		System.out.println("Deleting application '" + appName + "'");
	}


	/**
	 * Method listApplications.
	 * @param prevayler where we will execute the command
	 */
	private static void listApplications(Prevayler prevayler) {
		TreeMap sorted_map = new TreeMap(ObjectManager.getInstance());
		Set s = sorted_map.keySet();
		Iterator i = s.iterator();
		System.out.println("\n\nApplication List:");
		while (i.hasNext()) {
			System.out.println("  " + i.next());
		}
		System.out.println("");
	}


	/** Adds a new application to the system.
	 * @param prevayler where we will execute the command
	 * @param appName application name associated with the Jar files.
	 * @param rootDir root directory of the application.
	 */
	private static void addApplication(Prevayler prevayler, String appName, String rootDir) throws Exception {
		System.out.println("\nAdding application '" + appName + "'");

		if (null != ObjectManager.findByPrimaryKey(appName)) {
			System.out.println("ERROR: That application has already been processed - use the RELOAD or MERGE command.");
			return;
		}

		AddLookup lv = null;
		String key01 = null;

		DirectoryRecurser dr = new DirectoryRecurser();

		try {
			dr.setAppName(appName);
			dr.setRootDir(rootDir);
			dr.findFiles();
		} catch (JarSearcherException e) {
			System.out.println("JarSearcherException:AddApplication:ProcessRootDir: " + e.getMessage());
		}

		try {
			prevayler.executeCommand(new AddObject(appName, dr.getJarFiles()));
		} catch (AlreadyExistsException e) {
			// ignore this exception.
		}

	}

	private void verify(boolean condition, String message) {
		if (!condition) {
			throw new RuntimeException(message);
		}
	}

	private void out(Object obj) {
		System.out.println(obj);   //Uncomment this line to see what the test is doing.
	}

	private class PrevalenceFilter implements FilenameFilter {
		public boolean accept(File directory, String filename) {
			return filename.endsWith("commandLog")
				|| filename.endsWith("snapshot");
		}
	}

		/**
		 * Sets the isInteractive.
		 * @param isInteractive The isInteractive to set
		 */
		public void setIsInteractive(boolean isInteractive) {
			this.isInteractive = isInteractive;
		}

}
