package com.affy.jarsearcher;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;
import java.util.Map;
import java.util.jar.Attributes;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.JarInputStream;
import java.util.jar.Manifest;

/**
 * @author medined
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class JarSearcherWithHSQL {

	Connection conn = null;

	//use for SQL commands CREATE and SELECT
    public synchronized void query(String expression) throws SQLException {

        Statement st = null;
        ResultSet rs = null;

        st = conn.createStatement();            // statement objects can be reused with
                                                // repeated calls to execute but we
                                                // choose to make a new one each time
        rs = st.executeQuery(expression);       // run the query

        // do something with the result set.
        dump(rs);
        st.close();     // NOTE!! if you close a statement the associated ResultSet is
                        // closed too
                        // so you should copy the contents to some other object.
                        // the result set is invalidated also  if you recycle an Statement
                        // and try to execute some other query before the result set has been
                        // completely examined.
    }

	//use for SQL commands DROP and INSERT and UPDATE
    public synchronized void update(String expression) throws SQLException {

        Statement st = null;

        st = conn.createStatement();                // statements

        int i = st.executeUpdate(expression);       // run the query

        if (i == -1) {
            System.out.println("db error : " + expression);
        }

        st.close();
    }    // void update()

    public static void dump(ResultSet rs) throws SQLException {

        // the order of the rows in a cursor
        // are implementation dependent unless you use the SQL ORDER statement
        ResultSetMetaData meta   = rs.getMetaData();
        int               colmax = meta.getColumnCount();
        int               i;
        Object            o       = null;

        // the result set is a cursor into the data.  You can only
        // point to one row at a time
        // assume we are pointing to BEFORE the first row
        // rs.next() points to next row and returns true
        // or false if there is no next row, which breaks the loop 
        for (;rs.next();) {
            for (i = 0; i < colmax; ++i) {
                o = rs.getObject(i + 1);    // Is SQL the first column is indexed
                                            // with 1 not 0
                System.out.print(o.toString() + " ");
            }

            System.out.println(" ");
        }
    }
    
    public void start(String db_file_name_prefix) throws Exception {
		try {
        	Class.forName("org.hsqldb.jdbcDriver");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			throw e;
		}		

		try {
			conn = DriverManager.getConnection("jdbc:hsqldb:"
				+ db_file_name_prefix,  // filename
			    "sa",                   // username
			    "");			  	    // password
		} catch (SQLException e) {
			e.printStackTrace();
			throw e;
		}
    }
	
	public void stop() throws Exception {
		try {
			conn.close();
		} catch (SQLException e) {
			e.printStackTrace();
			throw e;
		}
	}

	public void execute(String jarFilename) {
/*
 *         try {

            //make an empty table
            //
            // by declaring the id column IDENTITY, the db will automatically
            // generate unique values for new rows- useful for row keys
            db.query(
                "CREATE TABLE sample_table ( id INTEGER IDENTITY, str_col VARCHAR(256), num_col INTEGER)");
        } catch (SQLException ex2) {

            //ignore
            //ex2.printStackTrace();  // second time we run program
            //  should throw execption since table
            // already there
            //
            // this will have no effect on the db
        }

        try {

            // add some rows - will create duplicates if run more then once
            // the id column is automatically generated
            db.update(
                "INSERT INTO sample_table(str_col,num_col) VALUES('Ford', 100)");
            db.update(
                "INSERT INTO sample_table(str_col,num_col) VALUES('Toyota', 200)");
            db.update(
                "INSERT INTO sample_table(str_col,num_col) VALUES('Honda', 300)");
            db.update(
                "INSERT INTO sample_table(str_col,num_col) VALUES('GM', 400)");

            // do a query
            db.query("SELECT * FROM sample_table WHERE num_col < 250");

            // at end of program
            db.shutdown();
        } catch (SQLException ex3) {
            ex3.printStackTrace();
        }
*/

        System.out.println("JarSearcher: starting.");

		JarInputStream jis = null;
		
		try {
			jis = new JarInputStream(new BufferedInputStream(new FileInputStream(jarFilename)));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		JarEntry je;
		
        try {
			while( ( je = jis.getNextJarEntry() ) != null ) {
			    System.out.println( je.getName() );
			    System.out.println( je.isDirectory() );
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) throws Exception {
		// D:\java\jwsdp-1_0_01\bin
		JarSearcherWithHSQL j = new JarSearcherWithHSQL();
		j.start("JARSEARCHER");
		j.execute("D:/java/jwsdp-1_0_01/bin/bootstrap.jar");
		j.stop();
	}
}
