package com.affy.jarsearcher.commands;

import com.affy.jarsearcher.ClassStore;
import com.affy.jarsearcher.ClassManager;
import com.affy.jarsearcher.exceptions.*;

import org.prevayler.Command;
import org.prevayler.PrevalentSystem;

import java.io.Serializable;
import java.util.HashMap;

/** Adds an Application to an ApplicationManager
 * @author Carlos Villela
 */
public final class AddClass implements Command {

	private ClassStore classStore;

	/**
	 * @see Command#execute(PrevalentSystem)
	 */
	public Serializable execute(PrevalentSystem sys) throws InvalidCommandException {
		//System.out.println("inside AddClass.execute");

		if (classStore == null)
			throw new InvalidCommandException("null classStore");

		try {
			if (false == classStore.validate()) {
				throw new InvalidCommandException("Invalid classStore object.");
			}
		} catch (InvalidClassException e) {
			throw new InvalidCommandException(e);
		}

		HashMap am = ClassManager.getInstance();
		
		ClassManager.put(classStore.getClassName(), classStore);

		return this;
	}

	/**
	 * Gets the class storage object.
	 * @return Returns a ClasStore object.
	 */
	public ClassStore getClassStore() {
		return classStore;
	}

	/**
	 * Sets the user.
	 * @param user The user to set
	 */
	public void setClassStore(ClassStore classStore) {
		this.classStore = classStore;
	}
}
