package com.affy.prevayler.commands;

import java.io.Serializable;
import java.util.HashMap;

import com.affy.prevayler.exceptions.AlreadyExistsException;
import com.affy.prevayler.util.ObjectManager;
import org.prevayler.Command;
import org.prevayler.PrevalentSystem;

/** Provides a lookup code for any Object. For example,
 * we can associate APP1 with "Ant v1.5" and APP2 with
 * "Apache v3.09". This class is intended to reduce the
 * memory needed to refer to Objects in Prevayler systems.
 * 
 * @author medined
 */
public class AddLookup implements Command {

	private String prefix = null;
	private String key    = null;
	private Object value  = null;

	public AddLookup() { }

	public AddLookup(Object o) { 
		init("A", o); 
	}

	public AddLookup(String prefix, Object o) { init(prefix, o); }

	public void init(String prefix, Object o) {
		this.prefix = prefix;
		this.key    = prefix + ObjectManager.getSize();
		this.value  = o;
	}

	public boolean validate() throws NullPointerException {
		if (getKey() == null)
			throw new NullPointerException("Null Key");

		if (getValue() == null)
			throw new NullPointerException("Null Value");

		return true;
	}

	/**
	 * @see Object#toString()
	 */
	public String toString() {
		return "LookupValue@"
			+ hashCode()
			+ "(key="
			+ getKey()
			+ " value="
			+ getValue()
			+ ");";
	}

	/**
	 * Returns the key.
	 * @return String
	 */
	public String getKey() {
		return key;
	}

	/**
	 * Returns the value.
	 * @return Object
	 */
	public Object getValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	public void setValue(Object value) {
		this.value = value;
	}


	/**
	 * @see Command#execute(PrevalentSystem)
	 */
	public Serializable execute(PrevalentSystem sys) throws AlreadyExistsException, NullPointerException {

		if (validate() && !ObjectManager.containsValue(getValue())) {
			ObjectManager.put(key, value);
		} else {
			throw new AlreadyExistsException("key [" + key + "] already exists.");
		}		
		return this;
	}

}
