package com.affy.prevayler.commands;

import java.io.Serializable;
import java.util.HashMap;

import com.affy.prevayler.exceptions.AlreadyExistsException;
import com.affy.prevayler.util.ObjectManager;

import org.prevayler.Command;
import org.prevayler.PrevalentSystem;


/** 
 */
public final class AddObject implements Command {

	private String key   = null;
	private Object value = null;

	public AddObject() { }
	
	public AddObject(String key, Object value) {
		this.key = key;
		this.value = value;
	}

	/**
	 * @see Command#execute(PrevalentSystem)
	 */
	public Serializable execute(PrevalentSystem sys) throws AlreadyExistsException, NullPointerException {

		if (key== null)
			throw new NullPointerException("Null key");
		if (value == null)
			throw new NullPointerException("Null value");

		if (null == ObjectManager.findByPrimaryKey(key)) {
			ObjectManager.put(key, value);
		} else {
			throw new AlreadyExistsException("key [" + key + "] already exists.");
		}
		
		return this;
	}

	/**
	 * Sets the value.
	 * @param value The user to set
	 */
	public void setValue(Object value) {
		this.value = value;
	}
	/**
	 * Sets the key.
	 * @param key The key to set
	 */
	public void setKey(String key) {
		this.key = key;
	}

}