package com.affy.prevayler.test;

import java.io.File;
import java.io.FilenameFilter;
import java.text.DecimalFormat;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeMap;

import org.prevayler.implementation.SnapshotPrevayler;

import com.affy.prevayler.commands.*;
import com.affy.prevayler.exceptions.AlreadyExistsException;
import com.affy.prevayler.util.*;

/**
 * @author David Medinets
 */
public class Main_SavingLookupValues {

	SnapshotPrevayler prevayler = null;

	/**
	 * Constructor.
	 */
	public Main_SavingLookupValues() { super(); }

	public static void main(String[] args) throws Exception {
		StopWatch.add("Main");
		Main_SavingLookupValues m = new Main_SavingLookupValues();
		m.process("CLEAR_DATABASE");
		System.out.println("Total Elapsed Time: " + StopWatch.elapsedTime("Main") + " ms.");
	}

	/**
	 * Method start.
	 */
	public void process(String options) throws Exception {
		System.out.println("Prevayler Test: Saving Lookup Values");

		if (options.equals("CLEAR_DATABASE")) {
			PrevaylerHelper.deletePrevalenceFiles(ObjectManager.prevalenceBase());
		}

		if (prevayler == null) {
			setPrevayler(new SnapshotPrevayler(new ObjectManager(), ObjectManager.prevalenceBase()));
		}

		String value01 = "Ant v1.5";
		String value02 = "Apache v3.09";

		AddLookup lv = null;
		String key01 = null;
		String key02 = null;
		
		// Add the Ant application as a lookup value to get a 
		// unique id.
		try {
			prevayler.executeCommand((lv = new AddLookup(value01)));
			key01 = lv.getKey();
		} catch (AlreadyExistsException e) {
			key01 = ObjectManager.findByValue(value01);
		}
		// add the inverse relationship so the key can be found 
		// from the application name. The assumption here is that
		// the application name are unique.
		try {
			prevayler.executeCommand(new AddString(value01, key01));
		} catch (AlreadyExistsException e) {
			// ignore this exception.
		}

		try {
			prevayler.executeCommand((lv = new AddLookup(value02)));
			key02 = lv.getKey();
		} catch (AlreadyExistsException e) {
			key02 = ObjectManager.findByValue(value01);
		}
		try {
			prevayler.executeCommand(new AddString(value02, key02));
		} catch (AlreadyExistsException e) {
			// ignore this exception.
		}

		System.out.println(ObjectManager.findByPrimaryKey(value01) + " --> " + ObjectManager.findByPrimaryKey(key01));
		System.out.println(ObjectManager.findByPrimaryKey(value02) + " --> " + ObjectManager.findByPrimaryKey(key02));

	}

	/**
	 * Method setPrevayler.
	 * @param prevayler
	 */
	public void setPrevayler(SnapshotPrevayler prevayler) {
		this.prevayler = prevayler;
	}

}
