package com.affy.prevayler.test;

import java.io.File;
import java.io.FilenameFilter;
import java.text.DecimalFormat;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.prevayler.implementation.SnapshotPrevayler;

import com.affy.prevayler.commands.AddObject;
import com.affy.prevayler.exceptions.AlreadyExistsException;
import com.affy.prevayler.util.ObjectManager;
import com.affy.prevayler.util.PrevaylerHelper;
import com.affy.prevayler.util.StopWatch;

/**
 * @author David Medinets
 */
public class Main_SavingObjects {

	SnapshotPrevayler prevayler = null;

	/**
	 * Constructor.
	 */
	public Main_SavingObjects() { super(); }

	public static void main(String[] args) throws Exception {
		StopWatch.add("Main");
		Main_SavingObjects m = new Main_SavingObjects();
		m.process("CLEAR_DATABASE");
		System.out.println("Total Elapsed Time: " + StopWatch.elapsedTime("Main") + " ms.");
	}

	/**
	 * Method start.
	 */
	public void process(String options) throws Exception {
		System.out.println("Prevayler Test: Saving Airports");

		if (options.equals("CLEAR_DATABASE")) {
			PrevaylerHelper.deletePrevalenceFiles(ObjectManager.prevalenceBase());
		}

		if (prevayler == null) {
			setPrevayler(new SnapshotPrevayler(new ObjectManager(), ObjectManager.prevalenceBase()));
		}
		
		Airport lax = new Airport("LAX");
		lax.setAirportName("Los Angeles International");
		lax.setCityName("Los Angeles, CA");
		lax.setCustomsFacilities(true);

	  	Airport iad = new Airport("IAD");
		iad.setAirportName("Washington Dulles");
		iad.setCityName("Washington, DC");
		iad.setCustomsFacilities(true);

		Airport npn = new Airport("NPN");
		npn.setAirportName("Williamsburg / Newport-News");
		npn.setCityName("Newport News, VA");
		npn.setCustomsFacilities(false);

 		Airport jfk = new Airport("JFK");
 		jfk.setAirportName("John F. Kennedy Intl");
		jfk.setCityName("New York City, NY");
		jfk.setCustomsFacilities(false);

  			try {
				prevayler.executeCommand(new AddObject(lax.getIataCode(), lax));
			} catch (AlreadyExistsException e) {
				System.out.println("Exception:AddObject:ExecuteCommand: " + e.getMessage());
			}
  			try {
				prevayler.executeCommand(new AddObject(iad.getIataCode(), iad));
			} catch (AlreadyExistsException e) {
				System.out.println("Exception:AddObject:ExecuteCommand: " + e.getMessage());
			}
  			try {
				prevayler.executeCommand(new AddObject(npn.getIataCode(), npn));
			} catch (AlreadyExistsException e) {
				System.out.println("Exception:AddObject:ExecuteCommand: " + e.getMessage());
			}
  			try {
				prevayler.executeCommand(new AddObject(jfk.getIataCode(), jfk));
			} catch (AlreadyExistsException e) {
				System.out.println("Exception:AddObject:ExecuteCommand: " + e.getMessage());
			}

		System.out.println("LAX equals " + ObjectManager.findByPrimaryKey("LAX"));
		System.out.println("IAD equals " + ObjectManager.findByPrimaryKey("IAD"));
		System.out.println("NPN equals " + ObjectManager.findByPrimaryKey("NPN"));
		System.out.println("JFK equals " + ObjectManager.findByPrimaryKey("JFK"));
	}

	/**
	 * Method setPrevayler.
	 * @param prevayler
	 */
	public void setPrevayler(SnapshotPrevayler prevayler) {
		this.prevayler = prevayler;
	}

}
