package com.affy.prevayler.test;

import java.io.File;
import java.io.FilenameFilter;
import java.text.DecimalFormat;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.prevayler.implementation.SnapshotPrevayler;

import com.affy.prevayler.commands.AddString;
import com.affy.prevayler.exceptions.AlreadyExistsException;
import com.affy.prevayler.util.ObjectManager;
import com.affy.prevayler.util.PrevaylerHelper;
import com.affy.prevayler.util.StopWatch;

/**
 * @author David Medinets
 */
public class Main_SavingStrings {

	SnapshotPrevayler prevayler = null;

	/**
	 * Constructor.
	 */
	public Main_SavingStrings() { super(); }

	/** This method exercises the class as a stand-alone application.
	 */
	public static void main(String[] args) throws Exception {
		StopWatch.add("Main");
		Main_SavingStrings m = new Main_SavingStrings();
		m.process("CLEAR_DATABASE");
		System.out.println("Total Elapsed Time: " + StopWatch.elapsedTime("Main") + " ms.");
	}

	/**
	 * Method process.
	 */
	public void process(String options) throws Exception {
		int numStrings = 2000;
		
		System.out.println("Prevayler Test: Saving Strings");

		if (options.equals("CLEAR_DATABASE")) {
			PrevaylerHelper.deletePrevalenceFiles(ObjectManager.prevalenceBase());
		}

		if (prevayler == null) {
			setPrevayler(new SnapshotPrevayler(new ObjectManager(), ObjectManager.prevalenceBase()));
		}
		
		StopWatch.add("Adding Strings");
		for (int i = 0; i < numStrings; i++) {
			try {
				prevayler.executeCommand(new AddString("David" + i, "Medinets" + i));
			} catch (AlreadyExistsException e) {
				// ignore this exception.
			}
		}
		System.out.println("Elapsed Time to Add " + numStrings + " Strings: " + StopWatch.elapsedTime("Adding Strings") + " ms.");

		for (int i = 0; i < Math.min(100, numStrings); i++) {
			System.out.println("David" + i + " equals " + ObjectManager.findByPrimaryKey("David"+i));
		}

	}

	/** When this class is invoked by another, that class
	 * probably has it's own Prevayler object already
	 * created.
	 * @param prevayler - the prevayler object to use.
	 */
	public void setPrevayler(SnapshotPrevayler prevayler) {
		this.prevayler = prevayler;
	}

}
