package com.affy.prevayler.util;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Iterator;
import java.util.Set;

import org.prevayler.implementation.SnapshotPrevayler;

/** This class deletes files from a Prevayler base directory. It
 * is mainly intended to be used during testing as a easy way
 * to delete the database. Here is a usage example:
 * 
 * private SnapshotPrevayler prevayler = null;
 * private final Set prevaylers = new HashSet();
 *
 * SnapshotPrevayler prevayler = new SnapshotPrevayler(new StringManager(), StringManager.prevalenceBase());
 * prevaylers.add(prevayler);
 *  
 * try {
 *     PrevaylerHelper.clearPrevalenceBase(prevaylers);
 * } catch (Exception e) {
 *     e.printStackTrace();
 * }
 *
 * @author medined
 *
 */
public class PrevaylerHelper {

	/* This method will be useful if the Prevayler implement
	 * exposes the snapshortDirectory.
	static public void clearPrevalenceBase(Set prevaylers) throws Exception {
		Iterator it = prevaylers.iterator();
		while (it.hasNext()) {
			SnapshotPrevayler sp = (SnapshotPrevayler)it.next();
			sp.takeSnapshot(); //Closes the open log file.
			deletePrevalenceFiles(sp.getSnapshotDirectory());
		}
		prevaylers.clear();
	}
	 */

	static public void deletePrevalenceFiles(String directoryName) {
		File directory = new File(directoryName);
		if(!directory.exists()) return;

		File[] files = directory.listFiles(new PrevalenceFilter());

		for(int i = 0; i < files.length; ++i){
//			out("Deleting: " + files[i]);
			verify(files[i].delete(), "Unable to delete " + files[i]);
		}
	}

	static private void verify(boolean condition, String message) {
		if (!condition) {
			throw new RuntimeException(message);
		}
	}

	static private class PrevalenceFilter implements FilenameFilter {
		public boolean accept(File directory, String filename) {
			return filename.endsWith("commandLog")
				|| filename.endsWith("snapshot");
		}
	}

}
